@@ -9,7 +9,7 @@ use warnings;
 
 use Scalar::Util qw[ blessed reftype ];
 use Digest::MD5;
-our $VERSION = '0.10';
+our $VERSION = '0.11';
 
 our @EXPORT = qw[ wrap_hash ];
 
@@ -133,7 +133,7 @@ sub import {
 
         # clean out known attributes
         delete @{$args}{
-            qw[ -base -as -class -lvalue -undef -exists -defined -new -copy -clone ]
+            qw[ -base -as -class -lvalue -undef -exists -defined -new -copy -clone -immutable -lockkeys ]
         };
 
         if ( keys %$args ) {
@@ -171,6 +171,7 @@ sub _build_class {
         autoload_attr   => '',
         validate_inline => 'exists $self->{\<<KEY>>}',
         validate_method => 'exists $self->{$key}',
+        set             => '$self->{q[\<<KEY>>]} = $_[0] if @_;',
         meta => [  map { ( qq[q($_) => q($attr->{$_}),] ) } keys %$attr ],
     );
 
@@ -201,6 +202,10 @@ sub _build_class {
         push @{ $dict{body} }, q[ sub <<DEFINED>> { defined $_[0]->{$_[1] } } ];
     }
 
+    if ( $attr->{-immutable} ) {
+        $dict{set} = 'Carp::croak( q[Modification of a read-only value attempted]) if @_;'
+    }
+
     my $class_template = <<'END';
 package <<CLASS>>;
 
@@ -231,7 +236,7 @@ our $accessor_template = q[
       Carp::croak( qq[Can't locate object method "\<<KEY>>" via package @{[ Scalar::Util::blessed( $self ) ]}] );
     }
 
-   $self->{q[\<<KEY>>]} = $_[0] if @_;
+   <<SET>>
 
    return $self->{q[\<<KEY>>]};
   }
@@ -335,6 +340,28 @@ sub _build_constructor {
         }
     };
 
+    $dict{lock} = do {
+        if ( $args->{-immutable} ) {
+            push @{ $dict{use} }, q[use Hash::Util ();];
+            'Hash::Util::lock_hash(%$hash)'
+        }
+        elsif ( defined $args->{-lockkeys} ) {
+
+            if ( 'ARRAY' eq ref $args->{-lockkeys} ) {
+                _croak( "-lockkeys: attribute name ($_) is not a legal Perl identifier" )
+                  for grep { $_ !~ PerlIdentifier } @{ $args->{-lockkeys} };
+
+                push @{ $dict{use} }, q[use Hash::Util ();];
+                'Hash::Util::lock_keys_plus(%$hash, qw{ ' . join( ' ', @{ $args->{-lockkeys} } ) . ' });'
+            }
+            elsif ( $args->{-lockkeys} ) {
+                push @{ $dict{use} }, q[use Hash::Util ();];
+                'Hash::Util::lock_keys(%$hash)'
+            }
+        }
+    };
+
+
     #<<< no tidy
     my $code = q[
     package <<PACKAGE>>;
@@ -353,6 +380,7 @@ sub _build_constructor {
       }
       <<COPY>>
       bless $hash, $class;
+      <<LOCK>>
     }
     1;
     ];
@@ -455,7 +483,7 @@ Hash::Wrap - create on-the-fly objects from hashes
 
 =head1 VERSION
 
-version 0.10
+version 0.11
 
 =head1 SYNOPSIS
 
@@ -498,8 +526,8 @@ With C<Hash::Wrap>:
   print $obj->a;
 
 Elements can be added or removed to the object and accessors will
-track them.  If the object should be immutable, use the lock routines
-in L<Hash::Util> on it.
+track them.  The object may be made immutable, or may have a restricted
+set of attributes.
 
 There are many similar modules on CPAN (see L<SEE ALSO> for comparisons).
 
@@ -656,6 +684,17 @@ is used. If a coderef, it will be called as
 
 By default, the object uses the hash directly.
 
+=item C<--immutable> => I<boolean>
+
+The object's attributes and values are locked and may not be altered. Note that this
+locks the underlying hash.
+
+=item C<--lockkeys> => I<boolean> | I<arrayref>
+
+If the value is I<true>, the object's attributes are restricted to the existing keys in the hash.
+If it is an array reference, it specifies which attributes are allowed, I<in addition to existing attributes>.
+The attribute's values are not locked.  Note that this locks the underlying hash.
+
 =back
 
 =head2 Accessors
@@ -1087,8 +1126,8 @@ __END__
 #pod   print $obj->a;
 #pod
 #pod Elements can be added or removed to the object and accessors will
-#pod track them.  If the object should be immutable, use the lock routines
-#pod in L<Hash::Util> on it.
+#pod track them.  The object may be made immutable, or may have a restricted
+#pod set of attributes.
 #pod
 #pod There are many similar modules on CPAN (see L<SEE ALSO> for comparisons).
 #pod
@@ -1245,6 +1284,17 @@ __END__
 #pod
 #pod By default, the object uses the hash directly.
 #pod
+#pod =item C<--immutable> => I<boolean>
+#pod
+#pod The object's attributes and values are locked and may not be altered. Note that this
+#pod locks the underlying hash.
+#pod
+#pod =item C<--lockkeys> => I<boolean> | I<arrayref>
+#pod
+#pod If the value is I<true>, the object's attributes are restricted to the existing keys in the hash.
+#pod If it is an array reference, it specifies which attributes are allowed, I<in addition to existing attributes>.
+#pod The attribute's values are not locked.  Note that this locks the underlying hash.
+#pod
 #pod =back
 #pod
 #pod =head2 Accessors
